# SetMethods - An ad-on R package for advanced QCA
# Ioana-Elena Oana and Carsten Schneider
# MMR Functions Test Script:


# Single Case MMR formulas:

# Typical cases:
# Filter: S > 0.5 & Y > 0.5 (S- sufficient term, Y- outcome)
################

fct <- round(seq(0.6, 1, 0.1), digits = 1) #Focal Conjunct
cct <- round(seq(0.6, 1, 0.1), digits = 1) #Complementary Conjunct
yt <-  round(seq(0.6, 1, 0.1), digits = 1) #Outcome
hd <- expand.grid(fct, cct, yt)
colnames(hd) <- c("fct","cct", "yt")
hd$st <- pmin(hd$fct, hd$cct)
hd[,1:4] <- round(hd[,1:4], digits = 1)

# Add columns with values in the formulas:

hd$f8a <- round(with(hd, (abs(yt-st) + (1-fct))), digits = 2)
hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

### Rank 1:
###########

hd1 <- subset(hd, (fct <= cct))

par(mfrow=c(1,1))
f8sub <- subset(hd1, (cct==1) & (st<= yt))
with(f8sub, plot(st, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                 main = paste("Typical Cases CCT = 1"), cex.main=1, 
                 sub = paste("(abs(yt-st)+(1-st))"), cex.sub=1))
abline(0, 1, lty = 5)
abline(v = 0.5, lty = 5)
abline(h = 0.5, lty = 5)
with(f8sub, text(st, yt, labels = f8laba, cex=1))

### Rank 2:
###########

hd2 <- subset(hd, (fct > cct))

par(mfrow=c(2,2))
for (i in fct) {
  if (i>0.6) {
    f8sub <- subset(hd2, (fct==i) & (cct==0.6) & (st <= yt))
    with(f8sub, plot(st, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = paste("Typical Cases CCT = 0.6; FCT =",i),
                     sub = paste("(abs(yt-st)+(1-st))")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(st, yt, labels = f8laba, cex = 1))
  }}


# Deviant consistency cases:
# Filter: S > 0.5 & Y < 0.5 (S- sufficient term, Y- outcome)
################

sdc <- round(seq(0.6, 1, 0.1), digits = 1) #Sufficient Term
ydc <-  round(seq(0, 0.4, 0.1), digits = 1) #Outcome
hd <- expand.grid(sdc, ydc)
colnames(hd) <- c("sdc", "ydc")
hd[,1:2] <- round(hd[,1:2], digits = 1)

# Add columns with values in the formulas:

hd$f8a <- round(with(hd, ((1-(sdc-ydc))+(1-sdc))), digits = 2)

hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

par(mfrow=c(3,3))
for (i in sdc) {
f8sub <- subset(hd, sdc==i)
with(hd, plot(sdc, ydc, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
              main = "Deviant Consistency Cases", 
              sub = paste("(1-(sdc-ydc))+(1-sdc)")))
abline(0, 1, lty = 5)
abline(v = 0.5, lty = 5)
abline(h = 0.5, lty = 5)
with(hd, text(sdc, ydc, labels = f8laba, cex = 1))
}

# Deviant coverage cases:
# Filter: (F < 0.5) & (Y > 0.5) (F- sufficient formula, Y- outcome)
################


# NB: This test is quite hard to do and visualize, because we are working with both
# sufficient formula values and TT values      

ydcv <- round(seq(0.6, 1, 0.1), digits = 1) #Outcome
fdcv <-  round(seq(0, 0.4, 0.1), digits = 1) #Sufficient formula
ttcv <- round(seq(0.6, 1, 0.1), digits = 1) #Truth table membership
hd <- expand.grid(fdcv, ydcv, ttcv)
colnames(hd) <- c("fdcv", "ydcv", "ttcv")
hd[,1:3] <- round(hd[,1:3], digits = 1)

# Add columns with values in the formulas:

hd$f8a <- round(with(hd, (abs(ydcv-ttcv))+(1-ttcv)), digits = 2)
hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

par(mfrow=c(3,3))
for (i in ttcv) {
  f8sub <- subset(hd, (ttcv == i))  
  with(f8sub, plot(fdcv, ydcv, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                   main = paste("Deviant Coverage TT=",i), 
                   sub = paste("(abs(ydcv-ttcv))+(1-ttcv)")))
  abline(0, 1, lty = 5)
  abline(v = 0.5, lty = 5)
  abline(h = 0.5, lty = 5)
  with(f8sub, text(fdcv, ydcv, labels = f8laba, cex = 1))
}

# Comparative MMR formulas:

# Typical-IIR cases:
################

fct <- round(seq(0.6, 1, 0.1), digits = 1) #Focal Conjunct Typical
cct <- round(seq(0.6, 1, 0.1), digits = 1) #Complementary Conjunct Typical
yt <-  round(seq(0.6, 1, 0.1), digits = 1) #Outcome Typical
fci <- round(seq(0, 0.4, 0.1), digits = 1) #Focal Conjunct
cci <- round(seq(0, 1, 0.1), digits = 1) #Complementary Conjunct IIR
yi <- round(seq(0, 0.4, 0.1), digits = 1) #Outcome IIR
hd <- expand.grid(fct, cct, yt, fci, cci, yi)
colnames(hd) <- c("fct", "cct", "yt", "fci", "cci", "yi")
hd[,1:6] <- round(hd[,1:6], digits = 1)

# Add columns with values in the formulas:


hd$f8a <- round(with(hd, ((1-(fct-fci))+ #big diff. in FC
                            (1-(yt-yi))+ #big diff in Y
                            abs(cct-cci)+ #small diff in complementary conj.
                            2*abs(yt-pmin(fct,cct))+ 
                            2*abs(yi-pmin(fci,cci)))) #small corridor for mechanism
                , digits = 2)

hd$f8laba <- as.character(hd$f8a)

# Plots with CCT and CCI fixed and varying FCI and YI:

# RANK 1:FCT<=CCT, CCI>0.5, FCI<=CCI
# IIR above the diagonal:
par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 1) & (fct <= yt))
    
    if (j>=i) {
      with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct == 1) & (cci == 1) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# IIR below the diagonal:
par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 1) & (fct <= yt))
    
    if (j<i) {
      with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct == 1) & (cci == 1) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# RANK 2: FCT<=CCT, CCI<0.5, , FCI<=CCI
# IIR above the diagonal

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 0.4) & (fct <= yt))
    
    if (j>=i) {
      with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct == 1) & (cci == 0.4) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# IIR below the diagonal

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 0.4) & (fct <= yt))
    
    if (j<i) {
      with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct == 1) & (cci == 0.4) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})


#RANK 3:FCT<=CCT, FCI < 0.5 & CCI < 0.5 & CCI <= FCI 
# (they are all above the diagonal as CCI is providing the min 
# for this rank and it is set to 0)

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 0) & (fct <= yt))
    
    if (j>=i) {
      with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct == 1) & (cci == 0) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
      points(0, j)}
  }})

# RANK 4: FCT>CCT, CCI>0.5 , FCI <= CCI
# IIR above the diagonal

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 1) & (cct <= yt))
    
    if (j>=i) {
      with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(fct == 1) & (cci == 0.4) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# IIR below the diagonal

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 1) & (cct <= yt))
    
    if (j<i) {
      with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(fct == 1) & (cci == 0.4) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# RANK 5: FCT>CCT, CCI<0.5, , FCI <= CCI
# IIR above the diagonal:

# NB. For cases in this rank, while CCI is held constant, 
# CCT is moving (compared to FCT as for the other).
# Since CCT is moving and CCI is constant, and YT is moving and YI is constant, 
# these two distances cancel each other out.

# For exemplification look at the subseted dataframes that 
# show how the two principle cancel 
# each other out and the graph is not able to capture this:

subtest <- subset(hd, (fci == c(0.3)) & (yi == c(0.4)) &
                    (fct == 1) & (cci == 0.4) & (cct <= yt))
View(subtest)

####
par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 0.4) & (cct <= yt))
    
    if (j>=i) {
      with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(fct == 1) & (cci == 0.4) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# IIR below the diagonal:

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 0.4) & (cct <= yt))
    
    if (j<i) {
      with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(fct == 1) & (cci == 0.4) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                       sub = paste("(fci == ", i,") & (yi ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
      points(i, j)}
  }})

# RANK 6: FCT>CCT, FCI < 0.5 & CCI < 0.5 & CCI <= FCI
# (all above the diagonal as CCI is the min and set to 0)

# NB. For cases in this rank, CCI is held constant while CCT moving 
# (compared to FCT as for the other ranks).
# Since CCT is moving and CCI is constant, and YT is moving and YI is constant, 
# these two distances cancel each other out.

# For exemplification look at the subseted dataframes that show how the two principle cancel 
# each other out and the graph is not able to capture this:

subtest <- subset(hd, (fci == c(0.3)) & (yi == c(0.4)) &
                    (fct == 1) & (cci == 0) & (cct <= yt))
View(subtest)

####

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 0) & (cct <= yt))
    
    
    with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = "(fct == 1) & (cci == 0) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                     sub = paste("(fci == ", i,") & (yi ==", j,")")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
    points(0, j)
  }})

# For the last two ranks we need to create data that has CCI <0.5 and FCI > 0.5:

fct <- round(seq(0.6, 1, 0.1), digits = 1)
cct <- round(seq(0.6, 1, 0.1), digits = 1)
yt <-  round(seq(0.6, 1, 0.1), digits = 1)
fci <- round(seq(0.6, 1, 0.1), digits = 1)
cci <- round(seq(0, 0.4, 0.1), digits = 1)
yi <- round(seq(0, 0.4, 0.1), digits = 1)
hd2 <- expand.grid(fct, cct, yt, fci, cci, yi)
colnames(hd2) <- c("fct", "cct", "yt", "fci", "cci", "yi")
hd2[,1:6] <- round(hd2[,1:6], digits = 1)

# Add columns with values in the formulas:

hd2$f8a <- round(with(hd2, ((1-(fct-fci))+ #big diff. in FC
                              (1-(yt-yi))+ #big diff in Y
                              abs(cct-cci)+ #small diff in complementary conj.
                              2*abs(yt-pmin(fct,cct))+ 
                              2*abs(yi-pmin(fci,cci)))) #small corridor for mechanism
                 , digits = 2)

hd2$f8laba <- as.character(hd2$f8a)


# Rank 7 (FCT<=CCT, CCI <0.5, FCI > 0.5):


par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd2, (fci == i) & (yi == j) &
                      (cct == 1) & (cci == 0) & (fct <= yt))
    
    
    with(f8sub, plot(fct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = "(cct == 1) & (cci == 0) & (fct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                     sub = paste("(fci == ", i,") & (yi ==", j,")")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(fct, yt, labels = f8laba, cex = 1))
    points(0, j)
  }})


# Rank 8 (FCT>CCT, CCI <0.5, FCI > 0.5):

# NB. For cases in this rank, while CCI is held constant while CCT moving 
# (compared to FCT as for the other ranks).
# Since CCT is moving and CCI is constant, and YT is moving and YI is constant, 
# these two distances cancel each other out.

par(mfrow=c(3,3))
suppressWarnings(for (i in fci) {
  for (j in yi) {
    f8sub <- subset(hd2, (fci == i) & (yi == j) &
                      (fct == 1) & (cci == 0) & (cct <= yt))
    
    with(f8sub, plot(cct, yt, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = "(fct == 1) & (cci == 0.4) & (cct <= yt)", #"1-(fct-fci))+ (1-(yt-yi))+(cct-cci)+2*(yt-pmin(fct,cct))+2*(yi-pmin(fci,cci)))",
                     sub = paste("(fci == ", i,") & (yi ==", j,")")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(cct, yt, labels = f8laba, cex = 1))
    points(0, j)
  }})


# Typical-Typical cases:
################

fct1 <- round(seq(0.6, 1, 0.1), digits = 1) # Focal Conjunct Case 1
cct1 <- round(seq(0.6, 1, 0.1), digits = 1) # Complementary Conjunct Case 1
yt1 <-  round(seq(0.6, 1, 0.1), digits = 1) # Outcome Case 1
fct2 <- round(seq(0.6, 1, 0.1), digits = 1) # Focal Conjunct Case 2
cct2 <- round(seq(0.6, 1, 0.1), digits = 1) # Complementary Conjunct Case 2
yt2 <- round(seq(0.6, 1, 0.1), digits = 1)  # Outcome Case 2
hd <- expand.grid(fct1, cct1, yt1, fct2, cct2, yt2)
colnames(hd) <- c("fct1", "cct1", "yt1", "fct2", "cct2", "yt2")
hd[,1:6] <- round(hd[,1:6], digits = 1)


# Filtering:
# Case 1 should be more typical than Case 2:
# (Ytyp1- Styp1)/Styp1 >= (Ytyp2- Styp2)/Styp2
# Cases should be above the diagonal:
# (Styp1 <= Ytyp1) & (Styp2 <= Ytyp2)

hd <- subset(hd, (pmin(fct1,cct1) <= yt1) & (pmin(fct2,cct2) <= yt2) & 
               ((yt1-pmin(fct1,cct1))/pmin(fct1,cct1) >= (yt2-pmin(fct2,cct2))/pmin(fct2,cct2))
             )


# Add columns with values in the formula:

hd$f8a <- round(with(hd, ((0.5-(fct1-fct2))+ #big diff. in FC
                            (0.5-(yt1-yt2))+ #big diff in Y
                            abs(cct1-cct2)+ #small diff in complementary conj.
                            2*abs(yt1-pmin(fct1,cct1))+ 
                            2*abs(yt2-pmin(fct2,cct2)))) #small corridor for mechanism
                , digits = 2)

hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

# 2nd filtering (1-1, 1-2, 2-1, 2-2): 
# typ1: FC <= CC
# typ2: FC > CC

# (RANK 1:FCT1<=CCT1,FCT2<=CCT2):

par(mfrow=c(3,3))
suppressWarnings(for (i in fct2) {
  for (j in yt2) {
    if (j>=i) {
      f8sub <- subset(hd, (fct2 == i) & (yt2 == j) &
                        (cct1 == 1) & (cct2 == 1))
      with(f8sub, plot(fct1, yt1, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct1 == 1) & (cct2 == 1)", 
                       sub = paste("(fct2 == ", i,") & (yt2 ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct1, yt1, labels = f8laba, cex = 1))
      points(i, j)
    }}})

# (RANK 2:FCT1<=CCT1,FCT2>CCT2):

par(mfrow=c(3,3))
suppressWarnings(for (i in fct2) {
  for (j in yt2) {
    if (j>=i) {
      f8sub <- subset(hd, (fct2 == i) & (yt2 == j) &
                        (cct1 == 1) & (cct2 == 0.6))
      with(f8sub, plot(fct1, yt1, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(cct1 == 1) & (cct2 == 0.6)", 
                       sub = paste("(fct2 == ", i,") & (yt2 ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(fct1, yt1, labels = f8laba, cex = 1))
      points(0.6, j)
    }}})

# (RANK 3:FCT1>CCT1,FCT2<=CCT2):

par(mfrow=c(3,3))
suppressWarnings(for (i in fct1) {
  for (j in yt1) {
    f8sub <- subset(hd, (fct1 == i) & (yt1 == j) &
                      (cct1 == 0.6) & (cct2 == 1))
    with(f8sub, plot(fct2, yt2, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = "(cct2 == 1) & (cct1 == 0.6)", 
                     sub = paste("(fct1 == ", i,") & (yt1 ==", j,")")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(fct2, yt2, labels = f8laba, cex = 1))
    points(0.6, j, cex=1)
  }})

# (RANK 4:FCT1>CCT1,FCT2>CCT2):

par(mfrow=c(3,3))
suppressWarnings(for (i in fct1) {
  for (j in yt1) {
    f8sub <- subset(hd, (fct1 == i) & (yt1 == j) &
                      (cct1 == 0.6) & (fct2 == 1))
    with(f8sub, plot(fct2, yt2, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                     main = "(cct1 == 0.6) & (fct2 == 1)",
                     sub = paste("(fct1 == ", i,") & (yt1 ==", j,")")))
    abline(0, 1, lty = 5)
    abline(v = 0.5, lty = 5)
    abline(h = 0.5, lty = 5)
    with(f8sub, text(cct2, yt2, labels = f8laba, cex = 1))
    points(0.6, j, cex=1)
  }})


# Typical-Deviant Consistency cases:
################

st <- round(seq(0.6, 1, 0.1), digits = 1) #Sufficient Term Typical
yt <-  round(seq(0.6, 1, 0.1), digits = 1) #Outcome Typical
sc <- round(seq(0.6, 1, 0.1), digits = 1) #Sufficient Term DCN
yc <- round(seq(0, 0.4, 0.1), digits = 1) #Outcome DCN

hd <- expand.grid(st, yt, sc, yc)
colnames(hd) <- c("st", "yt", "sc", "yc")
hd[,1:4] <- round(hd[,1:4], digits = 1)

# Add columns with values in the formulas:

hd$f8a <- round(with(hd, ((1-(yt-yc))+ #big diff in Y
                            abs(st-sc)+ #small diff in s.
                            (1-st)+
                            (1-sc))) #large s
                , digits = 2)

hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

# Plot:

par(mfrow=c(3,3))
suppressWarnings(for (i in st) {
  for (j in yt) {
    if (j>=i) {
      f8sub <- subset(hd, (st == i) & (yt == j))
      with(f8sub, plot(sc, yc, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = "(1-(yt-yc))+abs(st-sc)+(st+sc))", 
                       sub = paste("(st == ", i,") & (yt ==", j,")")))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(sc, yc, labels = f8laba, cex = 1))
      points(i, j)
    }}})


# Deviant Coverage-IIR cases:
################

# Assume they all are in the same TT Row:
# DCV: (F<0.5) & (Y>0.5) & (TT<=Y) 
# IIR: (S<0.5) & (Y<0.5)


fc <- round(seq(0, 0.4, 0.1), digits = 1) #Formula membership DCV
yc <-  round(seq(0.6, 1, 0.1), digits = 1) #Outcome DCV
ttc <- round(seq(0.6, 1, 0.1), digits = 1) #TT membership DCV
si <- round(seq(0, 0.4, 0.1), digits = 1) #Formula membership IIR
yi <- round(seq(0, 0.4, 0.1), digits = 1) #Outcome IIR
tti <- round(seq(0.6, 1, 0.1), digits = 1) #TT membership DCV

hd <- expand.grid(fc, yc, ttc, si, yi, tti)
colnames(hd) <- c("fc", "yc", "ttc", "si","yi","tti")
hd[,1:6] <- round(hd[,1:6], digits = 1)

# Add columns with values in the formulas:
hd$f8a <- round(with(hd, ((1-(yc-yi))+ #big diff in Y
                            abs(ttc-tti)+ #small diff in TT.
                            (1-ttc)+
                            (1-tti))) #large TT
                , digits = 2)

hd$f8laba <- as.character(hd$f8a)
hd$f8a<- round(hd$f8a, digits = 2)

# Plot:
# NB. This test is quite hard to visualize, 
# because we are working with formula values and TT values
# However, the formula values never enter into the calculation

# Formula DCV = 0:

par(mfrow=c(3,3))
suppressWarnings(for (i in ttc) {
  for (j in yc) {
    if (j>=i) {
      f8sub <- subset(hd, (ttc == i) & (yc == j) & (fc == 0) & (tti==1))
      with(f8sub, plot(si, yi, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = paste("(ttc == ", i,") & (yc ==", j,") & (fc = 0) & (tti=1)"), 
                       sub = "(1-(yc-yi))+abs(ttc-tti)+ (1-ttc)+ (1-tti))"))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(si, yi, labels = f8laba, cex = 1))
      points(0, j, cex=1)
    }}})

# Formula DCV = 0.4:

par(mfrow=c(3,3))
suppressWarnings(for (i in ttc) {
  for (j in yc) {
    if (j>=i) {
      f8sub <- subset(hd, (ttc == i) & (yc == j) & (fc == 0.4) & (tti==1))
      with(f8sub, plot(si, yi, type = "n", xlim = c(0, 1), ylim = c(0, 1), 
                       main = paste("(ttc == ", i,") & (yc ==", j,") & (fc = 0.4) & (tti=1)"),
                       sub = "(1-(yc-yi))+abs(ttc-tti)+ (1-ttc)+ (1-tti))"))
      abline(0, 1, lty = 5)
      abline(v = 0.5, lty = 5)
      abline(h = 0.5, lty = 5)
      with(f8sub, text(si, yi, labels = f8laba, cex = 1))
      points(0.4, j, cex=1)
    }}})
